@@ -15,12 +15,11 @@
 #    under the License.
 
 import functools
-import inspect
-import math
-import time
 
+import microversion_parse
 from oslo_log import log as logging
 from oslo_serialization import jsonutils
+from oslo_utils import encodeutils
 from oslo_utils import strutils
 import six
 import webob
@@ -43,11 +42,6 @@ _SUPPORTED_CONTENT_TYPES = (
     'application/vnd.openstack.compute+json',
 )
 
-_MEDIA_TYPE_MAP = {
-    'application/vnd.openstack.compute+json': 'json',
-    'application/json': 'json',
-}
-
 # These are typically automatically created by routes as either defaults
 # collection or member methods.
 _ROUTES_METHODS = [
@@ -70,9 +64,10 @@ DEFAULT_API_VERSION = "2.1"
 # name of attribute to keep version method information
 VER_METHOD_ATTR = 'versioned_methods'
 
-# Name of header used by clients to request a specific version
+# Names of headers used by clients to request a specific version
 # of the REST API
-API_VERSION_REQUEST_HEADER = 'X-OpenStack-Nova-API-Version'
+API_VERSION_REQUEST_HEADER = 'OpenStack-API-Version'
+LEGACY_API_VERSION_REQUEST_HEADER = 'X-OpenStack-Nova-API-Version'
 
 
 ENV_LEGACY_V2 = 'openstack.legacy_v2'
@@ -82,10 +77,6 @@ def get_supported_content_types():
     return _SUPPORTED_CONTENT_TYPES
 
 
-def get_media_map():
-    return dict(_MEDIA_TYPE_MAP.items())
-
-
 # NOTE(rlrossit): This function allows a get on both a dict-like and an
 # object-like object. cache_db_items() is used on both versioned objects and
 # dicts, so the function can't be totally changed over to [] syntax, nor
@@ -227,29 +218,30 @@ class Request(wsgi.Request):
 
     def set_api_version_request(self):
         """Set API version request based on the request header information."""
-        if API_VERSION_REQUEST_HEADER in self.headers:
-            hdr_string = self.headers[API_VERSION_REQUEST_HEADER]
-            # 'latest' is a special keyword which is equivalent to requesting
-            # the maximum version of the API supported
-            if hdr_string == 'latest':
-                self.api_version_request = api_version.max_api_version()
-            else:
-                self.api_version_request = api_version.APIVersionRequest(
-                    hdr_string)
+        hdr_string = microversion_parse.get_version(
+            self.headers, service_type='compute',
+            legacy_headers=[LEGACY_API_VERSION_REQUEST_HEADER])
 
-                # Check that the version requested is within the global
-                # minimum/maximum of supported API versions
-                if not self.api_version_request.matches(
-                        api_version.min_api_version(),
-                        api_version.max_api_version()):
-                    raise exception.InvalidGlobalAPIVersion(
-                        req_ver=self.api_version_request.get_string(),
-                        min_ver=api_version.min_api_version().get_string(),
-                        max_ver=api_version.max_api_version().get_string())
-
-        else:
+        if hdr_string is None:
             self.api_version_request = api_version.APIVersionRequest(
                 api_version.DEFAULT_API_VERSION)
+        elif hdr_string == 'latest':
+            # 'latest' is a special keyword which is equivalent to
+            # requesting the maximum version of the API supported
+            self.api_version_request = api_version.max_api_version()
+        else:
+            self.api_version_request = api_version.APIVersionRequest(
+                hdr_string)
+
+            # Check that the version requested is within the global
+            # minimum/maximum of supported API versions
+            if not self.api_version_request.matches(
+                    api_version.min_api_version(),
+                    api_version.max_api_version()):
+                raise exception.InvalidGlobalAPIVersion(
+                    req_ver=self.api_version_request.get_string(),
+                    min_ver=api_version.min_api_version().get_string(),
+                    max_ver=api_version.max_api_version().get_string())
 
     def set_legacy_v2(self):
         self.environ[ENV_LEGACY_V2] = True
@@ -542,90 +534,25 @@ class Resource(wsgi.Application):
     def deserialize(self, body):
         return JSONDeserializer().deserialize(body)
 
-    # NOTE(sdague): I didn't start the fire, however here is what all
-    # of this is about.
-    #
-    # In the legacy v2 code stack, extensions could extend actions
-    # with a generator that let 1 method be split into a top and
-    # bottom half. The top half gets executed before the main
-    # processing of the request (so effectively gets to modify the
-    # request before it gets to the main method).
-    #
-    # Returning a response triggers a shortcut to fail out. The
-    # response will nearly always be a failure condition, as it ends
-    # up skipping further processing one level up from here.
-    #
-    # This then passes on the list of extensions, in reverse order,
-    # on. post_process will run through all those, again with same
-    # basic logic.
-    #
-    # In tree this is only used in the legacy v2 stack, and only in
-    # the DiskConfig and SchedulerHints from what I can see.
-    #
-    # pre_process_extensions can be removed when the legacyv2 code
-    # goes away. post_process_extensions can be massively simplified
-    # at that point.
-    def pre_process_extensions(self, extensions, request, action_args):
-        # List of callables for post-processing extensions
-        post = []
-
-        for ext in extensions:
-            if inspect.isgeneratorfunction(ext):
-                response = None
-
-                # If it's a generator function, the part before the
-                # yield is the preprocessing stage
-                try:
-                    with ResourceExceptionHandler():
-                        gen = ext(req=request, **action_args)
-                        response = next(gen)
-                except Fault as ex:
-                    response = ex
-
-                # We had a response...
-                if response:
-                    return response, []
-
-                # No response, queue up generator for post-processing
-                post.append(gen)
-            else:
-                # Regular functions only perform post-processing
-                post.append(ext)
-
-        # None is response, it means we keep going. We reverse the
-        # extension list for post-processing.
-        return None, reversed(post)
-
-    def post_process_extensions(self, extensions, resp_obj, request,
-                                action_args):
+    def process_extensions(self, extensions, resp_obj, request,
+                           action_args):
         for ext in extensions:
             response = None
-            if inspect.isgenerator(ext):
-                # If it's a generator, run the second half of
-                # processing
-                try:
-                    with ResourceExceptionHandler():
-                        response = ext.send(resp_obj)
-                except StopIteration:
-                    # Normal exit of generator
-                    continue
-                except Fault as ex:
-                    response = ex
-            else:
-                # Regular functions get post-processing...
-                try:
-                    with ResourceExceptionHandler():
-                        response = ext(req=request, resp_obj=resp_obj,
-                                       **action_args)
-                except exception.VersionNotFoundForAPIMethod:
-                    # If an attached extension (@wsgi.extends) for the
-                    # method has no version match its not an error. We
-                    # just don't run the extends code
-                    continue
-                except Fault as ex:
-                    response = ex
+            # Regular functions get post-processing...
+            try:
+                with ResourceExceptionHandler():
+                    response = ext(req=request, resp_obj=resp_obj,
+                                   **action_args)
+            except exception.VersionNotFoundForAPIMethod:
+                # If an attached extension (@wsgi.extends) for the
+                # method has no version match its not an error. We
+                # just don't run the extends code
+                continue
+            except Fault as ex:
+                response = ex
 
-            # We had a response...
+            # We had a response return it, to exit early. This is
+            # actually a failure mode. None is success.
             if response:
                 return response
 
@@ -724,16 +651,12 @@ class Resource(wsgi.Application):
                      'context_project_id': context.project_id}
             return Fault(webob.exc.HTTPBadRequest(explanation=msg))
 
-        # Run pre-processing extensions
-        response, post = self.pre_process_extensions(extensions,
-                                                     request, action_args)
-
-        if not response:
-            try:
-                with ResourceExceptionHandler():
-                    action_result = self.dispatch(meth, request, action_args)
-            except Fault as ex:
-                response = ex
+        response = None
+        try:
+            with ResourceExceptionHandler():
+                action_result = self.dispatch(meth, request, action_args)
+        except Fault as ex:
+            response = ex
 
         if not response:
             # No exceptions; convert action_result into a
@@ -751,8 +674,8 @@ class Resource(wsgi.Application):
                 # Do a preserialize to set up the response object
                 if hasattr(meth, 'wsgi_code'):
                     resp_obj._default_code = meth.wsgi_code
-                # Process post-processing extensions
-                response = self.post_process_extensions(post, resp_obj,
+                # Process extensions
+                response = self.process_extensions(extensions, resp_obj,
                                                         request, action_args)
 
             if resp_obj and not response:
@@ -760,13 +683,21 @@ class Resource(wsgi.Application):
 
         if hasattr(response, 'headers'):
             for hdr, val in list(response.headers.items()):
-                # Headers must be utf-8 strings
-                response.headers[hdr] = utils.utf8(val)
+                if six.PY2:
+                    # In Py2.X Headers must be byte strings
+                    response.headers[hdr] = utils.utf8(val)
+                else:
+                    # In Py3.X Headers must be utf-8 strings
+                    response.headers[hdr] = encodeutils.safe_decode(
+                            utils.utf8(val))
 
             if not request.api_version_request.is_null():
                 response.headers[API_VERSION_REQUEST_HEADER] = \
+                    'compute ' + request.api_version_request.get_string()
+                response.headers[LEGACY_API_VERSION_REQUEST_HEADER] = \
                     request.api_version_request.get_string()
-                response.headers['Vary'] = API_VERSION_REQUEST_HEADER
+                response.headers.add('Vary', API_VERSION_REQUEST_HEADER)
+                response.headers.add('Vary', LEGACY_API_VERSION_REQUEST_HEADER)
 
         return response
 
@@ -1011,8 +942,16 @@ class Controller(object):
             # so later when we work through the list in order we find
             # the method which has the latest version which supports
             # the version requested.
-            # TODO(cyeoh): Add check to ensure that there are no overlapping
-            # ranges of valid versions as that is amibiguous
+            is_intersect = Controller.check_for_versions_intersection(
+                func_list)
+
+            if is_intersect:
+                raise exception.ApiVersionsIntersect(
+                    name=new_func.name,
+                    min_ver=new_func.start_version,
+                    max_ver=new_func.end_version,
+                )
+
             func_list.sort(key=lambda f: f.start_version, reverse=True)
 
             return f
@@ -1033,6 +972,36 @@ class Controller(object):
 
         return is_dict(body[entity_name])
 
+    @staticmethod
+    def check_for_versions_intersection(func_list):
+        """Determines whether function list contains version intervals
+        intersections or not. General algorithm:
+
+        https://en.wikipedia.org/wiki/Intersection_algorithm
+
+        :param func_list: list of VersionedMethod objects
+        :return: boolean
+        """
+        pairs = []
+        counter = 0
+
+        for f in func_list:
+            pairs.append((f.start_version, 1, f))
+            pairs.append((f.end_version, -1, f))
+
+        def compare(x):
+            return x[0]
+
+        pairs.sort(key=compare)
+
+        for p in pairs:
+            counter += p[1]
+
+            if counter > 1:
+                return True
+
+        return False
+
 
 class Fault(webob.exc.HTTPException):
     """Wrap webob.exc.HTTPException to provide API friendly response."""
@@ -1081,9 +1050,12 @@ class Fault(webob.exc.HTTPException):
 
         if not req.api_version_request.is_null():
             self.wrapped_exc.headers[API_VERSION_REQUEST_HEADER] = \
+                'compute ' + req.api_version_request.get_string()
+            self.wrapped_exc.headers[LEGACY_API_VERSION_REQUEST_HEADER] = \
                 req.api_version_request.get_string()
-            self.wrapped_exc.headers['Vary'] = \
-              API_VERSION_REQUEST_HEADER
+            self.wrapped_exc.headers.add('Vary', API_VERSION_REQUEST_HEADER)
+            self.wrapped_exc.headers.add('Vary',
+                                         LEGACY_API_VERSION_REQUEST_HEADER)
 
         self.wrapped_exc.content_type = 'application/json'
         self.wrapped_exc.charset = 'UTF-8'
@@ -1093,46 +1065,3 @@ class Fault(webob.exc.HTTPException):
 
     def __str__(self):
         return self.wrapped_exc.__str__()
-
-
-class RateLimitFault(webob.exc.HTTPException):
-    """Rate-limited request response."""
-
-    def __init__(self, message, details, retry_time):
-        """Initialize new `RateLimitFault` with relevant information."""
-        hdrs = RateLimitFault._retry_after(retry_time)
-        self.wrapped_exc = webob.exc.HTTPTooManyRequests(headers=hdrs)
-        self.content = {
-            "overLimit": {
-                "code": self.wrapped_exc.status_int,
-                "message": message,
-                "details": details,
-                "retryAfter": hdrs['Retry-After'],
-            },
-        }
-
-    @staticmethod
-    def _retry_after(retry_time):
-        delay = int(math.ceil(retry_time - time.time()))
-        retry_after = delay if delay > 0 else 0
-        headers = {'Retry-After': '%d' % retry_after}
-        return headers
-
-    @webob.dec.wsgify(RequestClass=Request)
-    def __call__(self, request):
-        """Return the wrapped exception with a serialized body conforming
-        to our error format.
-        """
-        user_locale = request.best_match_language()
-
-        self.content['overLimit']['message'] = \
-            i18n.translate(self.content['overLimit']['message'], user_locale)
-        self.content['overLimit']['details'] = \
-            i18n.translate(self.content['overLimit']['details'], user_locale)
-
-        content = JSONDictSerializer().serialize(self.content)
-        self.wrapped_exc.charset = 'UTF-8'
-        self.wrapped_exc.content_type = "application/json"
-        self.wrapped_exc.text = content
-
-        return self.wrapped_exc
