@@ -21,6 +21,7 @@ from nova.compute import vm_states
 import nova.conf
 from nova import db
 from nova import objects
+from nova import test
 from nova.tests.unit.compute import test_compute
 from nova.tests.unit.image import fake as fake_image
 from nova.tests import uuidsentinel as uuids
@@ -43,9 +44,20 @@ def _fake_resources():
 
 
 class ShelveComputeManagerTestCase(test_compute.BaseTestCase):
+    @mock.patch.object(nova.compute.manager.ComputeManager,
+                       '_terminate_volume_connections')
+    @mock.patch.object(nova.virt.fake.SmallFakeDriver, 'power_off')
+    @mock.patch.object(nova.virt.fake.SmallFakeDriver, 'snapshot')
+    @mock.patch.object(nova.compute.manager.ComputeManager, '_get_power_state')
+    @mock.patch.object(nova.compute.manager.ComputeManager,
+                       '_notify_about_instance_usage')
     @mock.patch('nova.compute.utils.notify_about_instance_action')
-    def _shelve_instance(self, shelved_offload_time,
-                         mock_notify, clean_shutdown=True):
+    def _shelve_instance(self, shelved_offload_time, mock_notify,
+                         mock_notify_instance_usage, mock_get_power_state,
+                         mock_snapshot, mock_power_off, mock_terminate,
+                         clean_shutdown=True):
+        mock_get_power_state.return_value = 123
+
         CONF.set_override('shelved_offload_time', shelved_offload_time)
         host = 'fake-mini'
         instance = self._create_fake_instance_obj(params={'host': host})
@@ -55,29 +67,6 @@ class ShelveComputeManagerTestCase(test_compute.BaseTestCase):
         instance.task_state = task_states.SHELVING
         instance.save()
 
-        self.mox.StubOutWithMock(self.compute, '_notify_about_instance_usage')
-        self.mox.StubOutWithMock(self.compute.driver, 'snapshot')
-        self.mox.StubOutWithMock(self.compute.driver, 'power_off')
-        self.mox.StubOutWithMock(self.compute, '_get_power_state')
-        self.mox.StubOutWithMock(self.compute.network_api,
-                                 'cleanup_instance_network_on_host')
-
-        self.compute._notify_about_instance_usage(self.context, instance,
-                'shelve.start')
-        if clean_shutdown:
-            self.compute.driver.power_off(instance,
-                                          CONF.shutdown_timeout,
-                                          self.compute.SHUTDOWN_RETRY_INTERVAL)
-        else:
-            self.compute.driver.power_off(instance, 0, 0)
-        self.compute._get_power_state(self.context,
-                instance).AndReturn(123)
-        if CONF.shelved_offload_time == 0:
-            self.compute.network_api.cleanup_instance_network_on_host(
-                self.context, instance, instance.host)
-        self.compute.driver.snapshot(self.context, instance, 'fake_image_id',
-                mox.IgnoreArg())
-
         tracking = {'last_state': instance.vm_state}
 
         def check_save(expected_task_state=None):
@@ -115,28 +104,63 @@ class ShelveComputeManagerTestCase(test_compute.BaseTestCase):
             else:
                 self.fail('Unexpected save!')
 
-        self.compute._notify_about_instance_usage(self.context,
-                                                  instance, 'shelve.end')
-        if CONF.shelved_offload_time == 0:
-            self.compute._notify_about_instance_usage(self.context, instance,
-                'shelve_offload.start')
-            self.compute.driver.power_off(instance, 0, 0)
-            self.compute._get_power_state(self.context,
-                                          instance).AndReturn(123)
-            self.compute._notify_about_instance_usage(self.context, instance,
-                                                      'shelve_offload.end')
-        self.mox.ReplayAll()
-
-        with mock.patch.object(instance, 'save') as mock_save:
+        with test.nested(
+                mock.patch.object(instance, 'save'),
+                mock.patch.object(self.compute.network_api,
+                                  'cleanup_instance_network_on_host')) as (
+            mock_save, mock_cleanup
+        ):
             mock_save.side_effect = check_save
             self.compute.shelve_instance(self.context, instance,
-                    image_id=image_id, clean_shutdown=clean_shutdown)
+                                         image_id=image_id,
+                                         clean_shutdown=clean_shutdown)
             mock_notify.assert_has_calls([
                 mock.call(self.context, instance, 'fake-mini',
                           action='shelve', phase='start'),
                 mock.call(self.context, instance, 'fake-mini',
                           action='shelve', phase='end')])
 
+        # prepare expect call lists
+        mock_notify_instance_usage_call_list = [
+            mock.call(self.context, instance, 'shelve.start'),
+            mock.call(self.context, instance, 'shelve.end')]
+        mock_power_off_call_list = []
+        mock_get_power_state_call_list = [
+            mock.call(self.context, instance)]
+        mock_cleanup_call_list = []
+
+        if clean_shutdown:
+            mock_power_off_call_list.append(
+                mock.call(instance, CONF.shutdown_timeout,
+                          self.compute.SHUTDOWN_RETRY_INTERVAL))
+        else:
+            mock_power_off_call_list.append(mock.call(instance, 0, 0))
+
+        if CONF.shelved_offload_time == 0:
+            mock_notify_instance_usage_call_list.extend([
+                mock.call(self.context, instance, 'shelve_offload.start'),
+                mock.call(self.context, instance, 'shelve_offload.end')])
+            mock_power_off_call_list.append(mock.call(instance, 0, 0))
+            mock_get_power_state_call_list.append(mock.call(self.context,
+                                                            instance))
+            # instance.host is replaced with host because
+            # original instance.host is clear after
+            # ComputeManager.shelve_instance execute with
+            # shelved_offload_time == 0
+            mock_cleanup_call_list.append(mock.call(self.context, instance,
+                                                    host))
+
+        mock_notify_instance_usage.assert_has_calls(
+            mock_notify_instance_usage_call_list)
+        mock_power_off.assert_has_calls(mock_power_off_call_list)
+        mock_cleanup.assert_has_calls(mock_cleanup_call_list)
+        mock_snapshot.assert_called_once_with(self.context, instance,
+                                              'fake_image_id', mock.ANY)
+        mock_get_power_state.assert_has_calls(mock_get_power_state_call_list)
+
+        if CONF.shelved_offload_time == 0:
+            self.assertTrue(mock_terminate.called)
+
     def test_shelve(self):
         self._shelve_instance(-1)
 
@@ -146,50 +170,67 @@ class ShelveComputeManagerTestCase(test_compute.BaseTestCase):
     def test_shelve_and_offload(self):
         self._shelve_instance(0)
 
-    def _shelve_offload(self, clean_shutdown=True):
+    @mock.patch.object(nova.virt.fake.SmallFakeDriver, 'power_off')
+    def test_shelve_offload(self, mock_power_off):
+        instance = self._shelve_offload()
+        mock_power_off.assert_called_once_with(instance,
+            CONF.shutdown_timeout, self.compute.SHUTDOWN_RETRY_INTERVAL)
+
+    @mock.patch.object(nova.virt.fake.SmallFakeDriver, 'power_off')
+    def test_shelve_offload_forced_shutdown(self, mock_power_off):
+        instance = self._shelve_offload(clean_shutdown=False)
+        mock_power_off.assert_called_once_with(instance, 0, 0)
+
+    @mock.patch.object(nova.compute.manager.ComputeManager,
+                       '_terminate_volume_connections')
+    @mock.patch.object(nova.compute.manager.ComputeManager,
+                       '_update_resource_tracker')
+    @mock.patch.object(nova.compute.manager.ComputeManager,
+                       '_get_power_state', return_value = 123)
+    @mock.patch.object(nova.compute.manager.ComputeManager,
+                       '_notify_about_instance_usage')
+    @mock.patch('nova.compute.utils.notify_about_instance_action')
+    def _shelve_offload(self, mock_notify, mock_notify_instance_usage,
+                        mock_get_power_state, mock_update_resource_tracker,
+                        mock_terminate, clean_shutdown=True):
         host = 'fake-mini'
         instance = self._create_fake_instance_obj(params={'host': host})
         instance.task_state = task_states.SHELVING
         instance.save()
         self.useFixture(utils_fixture.TimeFixture())
 
-        self.mox.StubOutWithMock(self.compute, '_notify_about_instance_usage')
-        self.mox.StubOutWithMock(self.compute.driver, 'power_off')
-        self.mox.StubOutWithMock(self.compute, '_get_power_state')
-        self.mox.StubOutWithMock(self.compute.network_api,
-                                 'cleanup_instance_network_on_host')
-        self.mox.StubOutWithMock(self.compute, '_update_resource_tracker')
-
-        self.compute._notify_about_instance_usage(self.context, instance,
-                'shelve_offload.start')
-        if clean_shutdown:
-            self.compute.driver.power_off(instance,
-                                          CONF.shutdown_timeout,
-                                          self.compute.SHUTDOWN_RETRY_INTERVAL)
-        else:
-            self.compute.driver.power_off(instance, 0, 0)
-        self.compute.network_api.cleanup_instance_network_on_host(
-                self.context, instance, instance.host)
-        self.compute._get_power_state(self.context,
-                instance).AndReturn(123)
-        self.compute._update_resource_tracker(self.context, instance)
-        self.compute._notify_about_instance_usage(self.context, instance,
-                'shelve_offload.end')
-        self.mox.ReplayAll()
-
         with mock.patch.object(instance, 'save'):
             self.compute.shelve_offload_instance(self.context, instance,
                                                  clean_shutdown=clean_shutdown)
+            mock_notify.assert_has_calls([
+                mock.call(self.context, instance, 'fake-mini',
+                          action='shelve_offload', phase='start'),
+                mock.call(self.context, instance, 'fake-mini',
+                          action='shelve_offload', phase='end')])
+
         self.assertEqual(vm_states.SHELVED_OFFLOADED, instance.vm_state)
         self.assertIsNone(instance.task_state)
+        self.assertTrue(mock_terminate.called)
 
-    def test_shelve_offload(self):
-        self._shelve_offload()
+        # prepare expect call lists
+        mock_notify_instance_usage_call_list = [
+            mock.call(self.context, instance, 'shelve_offload.start'),
+            mock.call(self.context, instance, 'shelve_offload.end')]
 
-    def test_shelve_offload_forced_shutdown(self):
-        self._shelve_offload(clean_shutdown=False)
+        mock_notify_instance_usage.assert_has_calls(
+            mock_notify_instance_usage_call_list)
+        # instance.host is replaced with host because
+        # original instance.host is clear after
+        # ComputeManager.shelve_offload_instance execute
+        mock_get_power_state.assert_called_once_with(
+            self.context, instance)
+        mock_update_resource_tracker.assert_called_once_with(self.context,
+                                                             instance)
 
-    def test_unshelve(self):
+        return instance
+
+    @mock.patch('nova.compute.utils.notify_about_instance_action')
+    def test_unshelve(self, mock_notify):
         instance = self._create_fake_instance_obj()
         instance.task_state = task_states.UNSHELVING
         instance.save()
@@ -220,10 +261,10 @@ class ShelveComputeManagerTestCase(test_compute.BaseTestCase):
         def fake_delete(self2, ctxt, image_id):
             self.deleted_image_id = image_id
 
-        def fake_claim(context, instance, limits):
+        def fake_claim(context, instance, node, limits):
             instance.host = self.compute.host
             requests = objects.InstancePCIRequests(requests=[])
-            return claims.Claim(context, instance,
+            return claims.Claim(context, instance, test_compute.NODENAME,
                                 self.rt, _fake_resources(),
                                 requests)
 
@@ -273,6 +314,13 @@ class ShelveComputeManagerTestCase(test_compute.BaseTestCase):
                 self.context, instance, image=image,
                 filter_properties=filter_properties,
                 node=node)
+
+            mock_notify.assert_has_calls([
+                mock.call(self.context, instance, 'fake-mini',
+                          action='unshelve', phase='start'),
+                mock.call(self.context, instance, 'fake-mini',
+                          action='unshelve', phase='end')])
+
         self.assertNotIn('shelved_at', instance.system_metadata)
         self.assertNotIn('shelved_image_id', instance.system_metadata)
         self.assertNotIn('shelved_host', instance.system_metadata)
@@ -287,7 +335,8 @@ class ShelveComputeManagerTestCase(test_compute.BaseTestCase):
         self.assertFalse(instance.auto_disk_config)
 
     @mock.patch('nova.utils.get_image_from_system_metadata')
-    def test_unshelve_volume_backed(self, mock_image_meta):
+    @mock.patch('nova.compute.utils.notify_about_instance_action')
+    def test_unshelve_volume_backed(self, mock_notify, mock_image_meta):
         instance = self._create_fake_instance_obj()
         node = test_compute.NODENAME
         limits = {}
@@ -329,9 +378,9 @@ class ShelveComputeManagerTestCase(test_compute.BaseTestCase):
                 mox.IgnoreArg(), do_check_attach=False).AndReturn('fake_bdm')
         self.compute.network_api.setup_instance_network_on_host(
                 self.context, instance, self.compute.host)
-        self.rt.instance_claim(self.context, instance, limits).AndReturn(
-                claims.Claim(self.context, instance, self.rt,
-                             _fake_resources(),
+        self.rt.instance_claim(self.context, instance, node, limits).AndReturn(
+                claims.Claim(self.context, instance, test_compute.NODENAME,
+                             self.rt, _fake_resources(),
                              objects.InstancePCIRequests(requests=[])))
         self.compute.driver.spawn(self.context, instance,
                 mox.IsA(objects.ImageMeta),
@@ -482,8 +531,10 @@ class ShelveComputeAPITestCase(test_compute.BaseTestCase):
             return metadata
 
         fake_image.stub_out_image_service(self)
-        self.stubs.Set(fake_image._FakeImageService, '__init__', fake_init)
-        self.stubs.Set(fake_image._FakeImageService, 'create', fake_create)
+        self.stub_out('nova.tests.unit.image.fake._FakeImageService.__init__',
+                      fake_init)
+        self.stub_out('nova.tests.unit.image.fake._FakeImageService.create',
+                      fake_create)
 
         self.compute_api.shelve(self.context, instance)
 
