@@ -13,9 +13,9 @@
 #    under the License.
 
 import datetime
-import uuid
 
 from oslo_serialization import jsonutils
+from oslo_utils import uuidutils
 
 from nova import objects
 from nova.objects import fields
@@ -58,7 +58,7 @@ def fake_db_instance(**updates):
     db_instance = {
         'id': 1,
         'deleted': False,
-        'uuid': str(uuid.uuid4()),
+        'uuid': uuidutils.generate_uuid(),
         'user_id': 'fake-user',
         'project_id': 'fake-project',
         'host': 'fake-host',
@@ -73,6 +73,7 @@ def fake_db_instance(**updates):
                   'flavor': flavorinfo,
                   'numa_topology': None,
                   'vcpu_model': None,
+                  'device_metadata': None,
                  },
         'tags': [],
         'services': []
@@ -85,7 +86,7 @@ def fake_db_instance(**updates):
             db_instance[name] = None
         elif field.default != fields.UnspecifiedDefault:
             db_instance[name] = field.default
-        elif name in ['flavor', 'ec2_ids']:
+        elif name in ['flavor', 'ec2_ids', 'keypairs']:
             pass
         else:
             raise Exception('fake_db_instance needs help with %s' % name)
@@ -120,6 +121,7 @@ def fake_instance_obj(context, obj_instance_class=None, **updates):
     inst = obj_instance_class._from_db_object(context,
                obj_instance_class(), fake_db_instance(**updates),
                expected_attrs=expected_attrs)
+    inst.keypairs = objects.KeyPairList(objects=[])
     if flavor:
         inst.flavor = flavor
     inst.old_flavor = None
