@@ -19,8 +19,8 @@ from nova.virt.ironic import ironic_states
 
 def get_test_validation(**kw):
     return type('interfaces', (object,),
-               {'power': kw.get('power', True),
-                'deploy': kw.get('deploy', True),
+               {'power': kw.get('power', {'result': True}),
+                'deploy': kw.get('deploy', {'result': True}),
                 'console': kw.get('console', True),
                 'rescue': kw.get('rescue', True)})()
 
@@ -45,7 +45,8 @@ def get_test_node(**kw):
                 'properties': kw.get('properties', {}),
                 'reservation': kw.get('reservation'),
                 'maintenance': kw.get('maintenance', False),
-                'network_provider': kw.get('network_provider'),
+                'network_interface': kw.get('network_interface'),
+                'resource_class': kw.get('resource_class'),
                 'extra': kw.get('extra', {}),
                 'updated_at': kw.get('created_at'),
                 'created_at': kw.get('updated_at')})()
@@ -57,6 +58,24 @@ def get_test_port(**kw):
                 'node_uuid': kw.get('node_uuid', get_test_node().uuid),
                 'address': kw.get('address', 'FF:FF:FF:FF:FF:FF'),
                 'extra': kw.get('extra', {}),
+                'internal_info': kw.get('internal_info', {}),
+                'portgroup_uuid': kw.get('portgroup_uuid'),
+                'created_at': kw.get('created_at'),
+                'updated_at': kw.get('updated_at')})()
+
+
+def get_test_portgroup(**kw):
+    return type('portgroup', (object,),
+               {'uuid': kw.get('uuid', 'deaffeed-1234-5678-9012-fedcbafedcba'),
+                'node_uuid': kw.get('node_uuid', get_test_node().uuid),
+                'address': kw.get('address', 'EE:EE:EE:EE:EE:EE'),
+                'extra': kw.get('extra', {}),
+                'internal_info': kw.get('internal_info', {}),
+                'properties': kw.get('properties', {}),
+                'mode': kw.get('mode', 'active-backup'),
+                'name': kw.get('name'),
+                'standalone_ports_supported': kw.get(
+                    'standalone_ports_supported', True),
                 'created_at': kw.get('created_at'),
                 'updated_at': kw.get('updated_at')})()
 
@@ -87,6 +106,9 @@ def get_test_flavor(**kw):
     flavor = {'name': kw.get('name', 'fake.flavor'),
               'extra_specs': kw.get('extra_specs', default_extra_specs),
               'swap': kw.get('swap', 0),
+              'root_gb': 1,
+              'memory_mb': 1,
+              'vcpus': 1,
               'ephemeral_gb': kw.get('ephemeral_gb', 0)}
     return objects.Flavor(**flavor)
 
@@ -105,6 +127,12 @@ class FakePortClient(object):
         pass
 
 
+class FakePortgroupClient(object):
+
+    def list(self, node=None, detail=False):
+        pass
+
+
 class FakeNodeClient(object):
 
     def list(self, detail=False):
@@ -116,10 +144,10 @@ class FakeNodeClient(object):
     def get_by_instance_uuid(self, instance_uuid, fields=None):
         pass
 
-    def list_ports(self, node_uuid):
+    def list_ports(self, node_uuid, detail=False):
         pass
 
-    def set_power_state(self, node_uuid, target):
+    def set_power_state(self, node_uuid, target, soft=False, timeout=None):
         pass
 
     def set_provision_state(self, node_uuid, target):
@@ -131,8 +159,18 @@ class FakeNodeClient(object):
     def validate(self, node_uuid):
         pass
 
+    def vif_attach(self, node_uuid, port_id):
+        pass
+
+    def vif_detach(self, node_uuid, port_id):
+        pass
+
+    def inject_nmi(self, node_uuid):
+        pass
+
 
 class FakeClient(object):
 
     node = FakeNodeClient()
     port = FakePortClient()
+    portgroup = FakePortgroupClient()
